/*
 *  This file is part of ixemul.library for the Amiga.
 *  Copyright (C) 1991, 1992  Markus M. Wild
 *  Portions Copyright (C) 1994 Rafael W. Luebbert
 *
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Library General Public
 *  License as published by the Free Software Foundation; either
 *  version 2 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Library General Public License for more details.
 *
 *  You should have received a copy of the GNU Library General Public
 *  License along with this library; if not, write to the Free
 *  Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *  $Id: ix_init.c,v 1.12 1994/07/11 22:16:47 rluebbert Exp $
 *
 *  $Log: ix_init.c,v $
 *  Revision 1.12  1994/07/11  22:16:47  rluebbert
 *  Removed arp.library from source - no longer needed.
 *
 *  Revision 1.11  1994/07/07  15:08:21  rluebbert
 *  10*NOFILE -> NOFILE
 *
 *  Revision 1.10  1994/07/07  15:03:00  rluebbert
 *  Removed ttyport dummy routines
 *
 *  Revision 1.9  1994/07/07  12:21:55  rluebbert
 *  *** empty log message ***
 *
 *  Revision 1.8  1994/07/07  11:44:26  rluebbert
 *  *** empty log message ***
 *
 *  Revision 1.7  1994/06/22  15:31:17  rluebbert
 *  Re-enable global environment variables
 *
 *  Revision 1.6  1994/06/19  15:12:59  rluebbert
 *  *** empty log message ***
 *
 *  Revision 1.4  1992/08/09  20:52:38  amiga
 *  change to new way of remembering open libraries
 *  get prepared to deal with ttys
 *
 *  Revision 1.3  1992/05/22  01:43:46  mwild
 *  initialize buddy allocator
 *
 * Revision 1.2  1992/05/18  00:49:07  mwild
 * changed async mp to be global
 *
 * Revision 1.1  1992/05/14  19:55:40  mwild
 * Initial revision
 *
 */

#define KERNEL
#include "ixemul.h"
#include "kprintf.h"

#include <exec/memory.h>
#include <dos/var.h>

#include <string.h>
#include <unistd.h>

/* not changed after the library is initialized */
struct ixemul_base		*ixemulbase = NULL;

struct ExecBase			*SysBase = NULL;
struct DosLibrary		*DOSBase = NULL;
struct MathIeeeSingBasBase	*MathIeeeSingBasBase = NULL;
struct MathIeeeDoubBasBase	*MathIeeeDoubBasBase = NULL;
struct MathIeeeDoubTransBase	*MathIeeeDoubTransBase = NULL;

/* global port for asynchronous packet notification */
struct MsgPort *ix_async_mp;

static struct
{
  void **base;
  char *name;
}
ix_libs[] =
{
  { (void **)&DOSBase, "dos.library" },
  { (void **)&MathIeeeSingBasBase, "mathieeesingbas.library" },
  { (void **)&MathIeeeDoubBasBase, "mathieeedoubbas.library" },
  { (void **)&MathIeeeDoubTransBase, "mathieeedoubtrans.library" },
  { NULL, NULL }
};

int open_libraries(void)
{
  int i;

  SysBase = *(struct ExecBase **)4;
  for (i = 0; ix_libs[i].base; i++)
    if (!(*(ix_libs[i].base) = (void *)OpenLibrary(ix_libs[i].name, 0)))
      {
	ix_panic("%s required!", ix_libs[i].name);
	return 0;
      }
  return 1;
}

void close_libraries(void)
{
  int i;
  
  for (i = 0; ix_libs[i].base; i++)
    if (*ix_libs[i].base)
      CloseLibrary(*ix_libs[i].base);
}

struct ixemul_base *ix_init (struct ixemul_base *ixbase)
{
  int i;
  char buf[256];

  ixemulbase = ixbase;
  if (!check_hardware())
    return 0;
  if (!open_libraries())
    {
      close_libraries();
      return 0;
    }

  ixbase->ix_file_tab = (struct file *)AllocMem (NOFILE * sizeof(struct file), MEMF_PUBLIC | MEMF_CLEAR);
  ixbase->ix_fileNFILE = ixbase->ix_file_tab + NOFILE;
  ixbase->ix_lastf = ixbase->ix_file_tab;

  /* Read the GMT offset. This environment variable is 5 bytes long. The
     first 4 form a long that contains the offset in seconds and the fifth
     byte is ignored. */
  if (GetVar("IXGMTOFFSET", buf, 6, GVF_BINARY_VAR) == 5 && IoErr() == 5)
    ix_set_gmt_offset(*((long *)buf));
  else
    ix_set_gmt_offset(0);

  if (GetVar(IX_ENV_SETTINGS, buf, sizeof(struct ix_settings) + 1, GVF_BINARY_VAR) == sizeof(struct ix_settings))
    ix_set_settings((struct ix_settings *)buf);
  else
    ix_set_settings(ix_get_default_settings());

  /* Set the hostname if the environment variable HOSTNAME exists. */
  if (GetVar("HOSTNAME", buf, 256, 0) > 0)
    sethostname(buf, strlen(buf));

  /* initialize the list structures for the allocator */
  init_buddy ();

  ix_async_mp = (struct MsgPort *) CreateInterruptPort (0, 0, mp_interrupt, 0);

  if (ixbase->ix_file_tab && ix_async_mp)
    {
      InitSemaphore (& ixbase->ix_semaph);

      configure_context_switch ();

      NewList ((struct List *) &ixbase->ix_socket_list);

      for (i = 0; i < IX_NUM_SLEEP_QUEUES; i++)
        NewList ((struct List *) &ixbase->ix_sleep_queues[i]);

      return ixbase;
    }
  if (ix_async_mp)
    DeleteInterruptPort (ix_async_mp);

  if (ixbase->ix_file_tab)
    FreeMem (ixbase->ix_file_tab, NOFILE * sizeof(struct file));
  else
    ix_panic ("out of memory");

  close_libraries();
  
  return 0;
}      

void ix_lock_base (void)
{
  u.u_oldmask = u.p_sigmask;
  u.p_sigmask = ~0;
  ObtainSemaphore (& ix.ix_semaph);
}

void ix_unlock_base (void)
{
  ReleaseSemaphore (& ix.ix_semaph);
  u.p_sigmask = u.u_oldmask;
}
